@@ -16,7 +16,7 @@ import { Logger } from '../util/logger'
 const CURRENT_URI = 'dummy-uri.sh'
 
 // if you add a .sh file to testing/fixtures, update this value
-const FIXTURE_FILES_MATCHING_GLOB = 16
+const FIXTURE_FILES_MATCHING_GLOB = 17
 
 const defaultConfig = getDefaultConfiguration()
 
@@ -144,7 +144,9 @@ describe('analyze', () => {
     expect(diagnostics3).toEqual([])
   })
 
-  it('handles tree-sitter crashes gracefully', async () => {
+  it('ensures tree-sitter does not crash', async () => {
+    // newer version of tree-sitter will crash on this file and never respond
+    // to new queries.
     const analyzer = await getAnalyzer({ runBackgroundAnalysis: false })
 
     // Parse the file
@@ -153,11 +155,6 @@ describe('analyze', () => {
       document: FIXTURE_DOCUMENT.CRASH,
     })
 
-    expect(loggerWarn).toHaveBeenCalled()
-    expect(loggerWarn.mock.calls).toEqual([
-      [expect.stringContaining('Tree sitter crashed while parsing')],
-    ])
-
     const result = analyzer.findAllSourcedUris({ uri: FIXTURE_URI.CRASH })
     expect(result).toEqual(new Set([]))
   })
@@ -465,6 +462,23 @@ describe('findDeclarationsMatchingWord', () => {
           },
           "name": "npm_config_loglevel",
         },
+        {
+          "kind": 13,
+          "location": {
+            "range": {
+              "end": {
+                "character": 26,
+                "line": 97,
+              },
+              "start": {
+                "character": 0,
+                "line": 97,
+              },
+            },
+            "uri": "file://__REPO_ROOT_FOLDER__/testing/fixtures/renaming.sh",
+          },
+          "name": "npm_config_loglevel",
+        },
       ]
     `)
 
